Imports System
Imports System.IO
Imports System.Collections
Imports System.Collections.Generic
Imports System.Text
Imports System.Runtime.InteropServices
Imports System.Windows.Forms
Imports System.Threading

<ComVisible(False)> _
Public Class GlobalData

    Private Shared _instance As GlobalData
    Private lastOutputPath_ As String

    Public Sub New()
        MyBase.New()

        Dim outputPath As String = Path.Combine( _
            Environment.GetFolderPath(Environment.SpecialFolder.InternetCache), "Low")
        outputPath = Path.Combine(outputPath, "DownloadManager")
        If (Not Directory.Exists(outputPath)) Then
            Directory.CreateDirectory(outputPath)
        End If

        Me.lastOutputPath_ = outputPath
    End Sub

    Public Shared ReadOnly Property Instance() As GlobalData
        Get
            If (GlobalData._instance Is Nothing) Then
                GlobalData._instance = New GlobalData()
            End If
            Return GlobalData._instance
        End Get
    End Property

    Public Property LastOutputPath() As String
        Get
            Return lastOutputPath_
        End Get

        Set(ByVal value As String)
            lastOutputPath_ = value
        End Set
    End Property

    <DllImport("kernel32.dll")> _
    Private Shared Function GetCurrentThreadId() As Integer
    End Function


    Public Sub SaveLinks(ByVal list As TreeView)

        Monitor.Enter(Instance)

        Try

            If (list.Nodes.Count > 0) Then

                Dim outputFile As String = String.Empty

                If String.IsNullOrEmpty(LastOutputPath) Then
                    Dim uri_ As Uri = New Uri(Instance.GetType().Assembly.CodeBase)
                    Dim outputPath As String = Path.GetDirectoryName(uri_.LocalPath)
                    outputFile = Path.Combine(outputPath, "dmvb.cfg")
                Else
                    outputFile = Path.Combine(LastOutputPath, "dmvb.cfg")
                End If

                Dim allowSaveLinks As Boolean = True
                Dim module_ As IEModule = AddinExpress.IE.ADXIEModule.CurrentInstance.Item(GetCurrentThreadId())

                If ((Not (module_ Is Nothing)) And (module_.IEVersion >= 7)) Then
                    Dim papi As AddinExpress.IE.ADXIEProtectedModeAPI = New AddinExpress.IE.ADXIEProtectedModeAPI()
                    allowSaveLinks = Not papi.IsProtectedModeOn()
                End If

                If (allowSaveLinks) Then

                    Using sw As StreamWriter = New StreamWriter(outputFile, True)

                        Dim i As Integer
                        Dim checked_ As Boolean

                        For i = 0 To list.Nodes.Count - 1
                            checked_ = list.Nodes.Item(i).Checked
                            sw.WriteLine(Convert.ToInt32(checked_).ToString() + "|" + (list.Nodes.Item(i).Tag).ToString())
                        Next

                    End Using
                End If

            End If

        Finally
            Monitor.Exit(Instance)
        End Try
    End Sub

    Public Sub LoadLinks(ByVal list As TreeView)

        Monitor.Enter(Instance)

        Try
            Dim outputFile As String = String.Empty

            If String.IsNullOrEmpty(LastOutputPath) Then
                Dim uri_ As Uri = New Uri(Instance.GetType().Assembly.CodeBase)
                Dim outputPath As String = Path.GetDirectoryName(uri_.LocalPath)
                outputFile = Path.Combine(outputPath, "dmvb.cfg")
            Else
                outputFile = Path.Combine(LastOutputPath, "dmvb.cfg")
            End If

            If (File.Exists(outputFile)) Then
                Try
                    list.BeginUpdate()

                    Using sr As StreamReader = New StreamReader(outputFile)
                        Dim key As String
                        Dim line As String
                        Dim parts As String()
                        Dim node As TreeNode

                        line = sr.ReadLine()

                        While Not (line Is Nothing)
                            parts = line.Split("|")

                            If (parts.Length = 2) Then
                                key = Path.GetFileName(parts(1))
                                node = list.Nodes.Add(key, key)
                                node.Tag = parts(1)
                                If (parts(0).Trim() = "1") Then
                                    node.Checked = True
                                End If

                                If (list.Nodes.Count = 1) Then
                                    list.SelectedNode = list.TopNode
                                End If
                            End If

                            line = sr.ReadLine()

                        End While

                    End Using

                    Try
                        File.Delete(outputFile)
                    Catch

                    End Try

                Finally
                    list.EndUpdate()
                End Try

            End If

        Finally
            Monitor.Exit(Instance)
        End Try

    End Sub

End Class

