Imports System
Imports System.Drawing
Imports System.ComponentModel
Imports System.Collections.Generic
Imports System.Diagnostics
Imports System.Text
Imports System.Windows.Forms
Imports System.Runtime.InteropServices

<StructLayout(LayoutKind.Explicit), ComVisible(False)> _
Public Structure RECT
    <FieldOffset(0)> _
    Public Left As Integer

    <FieldOffset(4)> _
    Public Top As Integer

    <FieldOffset(8)> _
    Public Right As Integer

    <FieldOffset(12)> _
    Public Bottom As Integer

    Public Sub New(ByVal left_ As Integer, ByVal top_ As Integer, ByVal right_ As Integer, ByVal bottom_ As Integer)
        Left = left_
        Top = top_
        Right = right_
        Bottom = bottom_
    End Sub

    Public Sub New(ByVal rect As Rectangle)
        Left = rect.Left
        Top = rect.Top
        Right = rect.Right
        Bottom = rect.Bottom
    End Sub

    Public Function ToRectangle() As Rectangle
        Return New Rectangle(Left, Top, Right, Bottom - 1)
    End Function

End Structure

<ComVisible(False)> _
Public Delegate Function DllIsThemeActive() As Boolean

<ComVisible(False)> _
Public Delegate Sub DllDrawThemeParentBackground(ByVal hWnd As IntPtr, ByVal hDC As IntPtr, ByRef rect As RECT)

Public Class TransparentLabel
    Inherits System.Windows.Forms.Label

    Private hThemeModule As IntPtr = IntPtr.Zero
    Private IsThemeActive As DllIsThemeActive = Nothing
    Private DrawThemeParentBackground As DllDrawThemeParentBackground = Nothing

    <DllImport("kernel32.dll")> _
    Private Shared Function LoadLibrary(ByVal lpFileName As String) As IntPtr
    End Function

    <DllImport("kernel32.dll", CharSet:=CharSet.Ansi)> _
    Private Shared Function GetProcAddress(ByVal hModule As IntPtr, ByVal lpProcName As String) As IntPtr
    End Function

    Public Sub New()
        MyBase.New()

        If (Not Me.DesignMode) Then
            hThemeModule = LoadLibrary("UxTheme.dll")
            If (hThemeModule <> IntPtr.Zero) Then

                Dim dllIsThemeActive_ As IntPtr = _
                    GetProcAddress(hThemeModule, "IsThemeActive")

                If (dllIsThemeActive_ <> IntPtr.Zero) Then
                    IsThemeActive = _
                        CType(Marshal.GetDelegateForFunctionPointer( _
                        dllIsThemeActive_, System.Type.GetType("DownloadManager.DllIsThemeActive")), DllIsThemeActive)
                End If

                Dim dllDrawThemeParentBackground_ As IntPtr = _
                    GetProcAddress(hThemeModule, "DrawThemeParentBackground")

                If (dllDrawThemeParentBackground_ <> IntPtr.Zero) Then
                    DrawThemeParentBackground = _
                       CType(Marshal.GetDelegateForFunctionPointer( _
                        dllDrawThemeParentBackground_, System.Type.GetType("DownloadManager.DllDrawThemeParentBackground")), DllDrawThemeParentBackground)
                End If

                Application.EnableVisualStyles()
                Application.DoEvents()
            End If
        End If

        InitializeComponent()
    End Sub

    Protected Overrides Sub OnPaintBackground(ByVal e As PaintEventArgs)
        If (hThemeModule <> IntPtr.Zero And (Not DesignMode)) Then
            If (IsThemeActive()) Then
                Dim hdc As IntPtr = IntPtr.Zero
                Dim rect As RECT = New RECT(0, 0, Width, Height)

                Try
                    hdc = e.Graphics.GetHdc()
                    DrawThemeParentBackground(Me.Handle, hdc, rect)
                Finally
                    If (hdc <> IntPtr.Zero) Then
                        e.Graphics.ReleaseHdc(hdc)
                    End If
                End Try

                Return
            End If
        End If
        MyBase.OnPaintBackground(e)
    End Sub

End Class
